# coding: utf-8
''' 
 *
 *  Copyright (C) 2013  André Luís da Costa, FEEC, Unicamp
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
'''

import threading
from PySide.QtCore import *
from PySide.QtGui import *
from PySide.QtOpenGL import *
from model import *
#from control import inv
import sys
import math
import numpy as np
from scipy.interpolate import UnivariateSpline
import time

try:
	import OpenGL
	import OpenGL.GL as GL
	import OpenGL.arrays
except ImportError:
	app = QApplication(sys.argv)
	QMessageBox.critical(None, "Visualizador de volumes",
						"PyOpenGL must be installed to run this example.",
						QMessageBox.Ok | QMessageBox.Default,
						QMessageBox.NoButton)
	sys.exit(1)

class Volume_Filter_Controler(QFrame):
	def __init__(self, canvas):
		super(Volume_Filter_Controler, self).__init__(None)
		self._canvas = canvas
		self._node_count = QLabel(u'Indefinido')
		self._volume_min = QSpinBox()
		self._volume_max = QSpinBox()
		self._volume_min.setRange(1,99999999)
		self._volume_max.setRange(1,99999999)
		self._volume_max.setValue(99999999)
		self._btn_exec = QPushButton(u'Aplicar filtro')
		layer = QFormLayout()
		layer.addRow(u'Volume mínimo', self._volume_min)
		layer.addRow(u'Volume máximo', self._volume_max)
		layer.addRow(u'Número de nós', self._node_count)
		layer.addRow(self._btn_exec)
		self.setLayout(layer)
		self._btn_exec.clicked.connect(self.execute_filter)
	
	def execute_filter(self):
		n = self._canvas.maxtreeFilter(self._volume_min.value(), self._volume_max.value())
		self._node_count.setText('%d'%n)
	
class MainWindow(QMainWindow):
	def __init__(self,  parent=None):
		from control import Chieftain
		super(MainWindow, self).__init__(parent)
		self._curve_editor = GLCurveEditor()
		self._canvas = GL3DViewer(mainwindow=self, sharedWidget=self._curve_editor)
		self._dock_curve_editor = QDockWidget(self.tr("Opacidade e Falsa Cor"))
		self._dock_curve_editor.setMinimumWidth(300)
		self._dock_curve_editor.setMaximumWidth(400)
		self._dock_curve_editor.setMinimumHeight(200)
		self._dock_curve_editor.setMaximumHeight(300)
		self._dock_curve_editor.setAllowedAreas(Qt.LeftDockWidgetArea | Qt.RightDockWidgetArea)
		self._dock_curve_editor.setFeatures(QDockWidget.DockWidgetFloatable | QDockWidget.DockWidgetMovable)
		self._dock_curve_editor.setWidget(self._curve_editor)
		
		self._filter_controls = Volume_Filter_Controler(self._canvas)
		self._filter_controls.setEnabled(False)
		self._dock_volume_properties = QDockWidget(self.tr("Filtro conexo"))
		self._dock_volume_properties.setMinimumWidth(300)
		self._dock_volume_properties.setMaximumWidth(400)
		self._dock_volume_properties.setMinimumHeight(300)
		self._dock_volume_properties.setAllowedAreas(Qt.LeftDockWidgetArea | Qt.RightDockWidgetArea)
		self._dock_volume_properties.setFeatures(QDockWidget.DockWidgetFloatable | QDockWidget.DockWidgetMovable)
		self._dock_volume_properties.setWidget(self._filter_controls)
		
		self._chieftain = Chieftain.get()
		self._chieftain._main = self
		self._chieftain._currentChanged.connect(self.__currentChanged)

		#self.curFile = ''
		self.createActions()
		self.createMenus()
		self.createToolBars()
		self.createStatusBar()
		#self.textEdit.document().contentsChanged.connect(self.documentWasModified)

		#self.setCurrentFile('')
		self.setUnifiedTitleAndToolBarOnMac(True)
		self.setWindowTitle("Visualizador [*]")
		self._void = QFrame(self)
		self.__currentChanged()
	
	def createActions(self):
		self.newAct = QAction(QIcon('images/new.png'), "&New",
			self, shortcut=QKeySequence.New,
			statusTip="Create a new dataset",  triggered=self._chieftain.new)

		self.openAct = QAction(QIcon('images/open.png'), "&Open",
			self, shortcut=QKeySequence.Open,
			statusTip="Open a stored dataset",  triggered=self._chieftain.open)

		self.closeAct = QAction(QIcon('images/close.png'), "&Close",
			self, shortcut=QKeySequence.Close,
			statusTip="Close current dataset",  triggered=self._chieftain.closeCurrent)

		self.saveAct = QAction(QIcon('images/save.png'), "&Save",
			self, shortcut=QKeySequence.Save,
			statusTip="Save current dataset",  triggered=self._chieftain.saveCurrent)

		self.saveasAct = QAction(QIcon('images/saveas.png'), "S&ave as",
			self, shortcut=QKeySequence.SaveAs, 
			statusTip="Save current dataset as",  triggered=self._chieftain.saveAs)

		self.saveallAct = QAction(QIcon('images/saveall.png'), "Sa&ve all",
			self, 
			statusTip="Save all datasets",  triggered=self._chieftain.saveAll)

		self.exitAct = QAction(QIcon('images/exit.png'), "E&xit", self, shortcut=QKeySequence.Quit, 
			statusTip="Close ORC-Bench application",  triggered=self._chieftain.closeAll)

		self.aboutAct = QAction("&About", self,
			statusTip="Show the application's About box",
			triggered=self.about)

		self.zoomIn = QAction("&Ampliar",
			self,
			statusTip="Ampliar a imagem",  triggered=self._chieftain.zoomIn)

		self.zoomOut = QAction("&Reduzir",
			self,
			statusTip="Reduzir a imagem",  triggered=self._chieftain.zoomOut)
		
		#self.cutAct.setEnabled(False)
		#self.copyAct.setEnabled(False)
		#self.textEdit.copyAvailable.connect(self.cutAct.setEnabled)
		#self.textEdit.copyAvailable.connect(self.copyAct.setEnabled)

	def documentWasModified(self):
		pass

	def createMenus(self):
		self.fileMenu = self.menuBar().addMenu("&File")
		#self.fileMenu.addAction(self.newAct)
		self.fileMenu.addAction(self.openAct)
		#self.fileMenu.addAction(self.closeAct)
		#self.fileMenu.addSeparator()
		#self.fileMenu.addAction(self.saveAct)
		#self.fileMenu.addAction(self.saveasAct)
		#self.fileMenu.addAction(self.saveallAct)
		self.fileMenu.addSeparator()
		self.fileMenu.addAction(self.exitAct) 

		self.menuBar().addSeparator()

		self.helpMenu = self.menuBar().addMenu("&Help")
		self.helpMenu.addAction(self.aboutAct)

	def createToolBars(self):
		self.fileToolBar = self.addToolBar("File")
		#self.fileToolBar.addAction(self.newAct)
		self.fileToolBar.addAction(self.openAct)
		self.fileToolBar.addAction(self.exitAct)
		#self.fileToolBar.addSeparator()
		#self.fileToolBar.addAction(self.zoomIn)
		#self.fileToolBar.addAction(self.zoomOut)
		#self.fileToolBar.addAction(self.saveAct)
		#self.fileToolBar.addAction(self.saveasAct)
		#self.fileToolBar.addAction(self.saveallAct)

	def createStatusBar(self):
		self.statusBar().showMessage("Ready")

	def newFile(self):
		pass

	def about(self):
		QMessageBox.about(self, "About Application",
				"The <b>Application</b> example demonstrates how to write "
				"modern GUI applications using Qt, with a menu bar, "
				"toolbars, and a status bar.")
	
	def __currentChanged(self):
		current = self._chieftain.getCurrent()
		if current==None:
			self.closeAct.setEnabled(False)
			self.setCentralWidget(self._void)
		else:
			self.closeAct.setEnabled(True)
			self._canvas.setVolume(current)
			self.addDockWidget(Qt.RightDockWidgetArea, self._dock_curve_editor)
			self.addDockWidget(Qt.RightDockWidgetArea, self._dock_volume_properties)
			self.setCentralWidget(self._canvas)
			threading.Thread(target=self._canvas.enableMaxtree).start()

class GL3DViewer(QGLWidget):
	def __init__(self, parent=None, mainwindow = None, sharedWidget=None):
		QGLWidget.__init__(self, parent, shareWidget = sharedWidget)
		self._mainwindow = mainwindow
		self.sharedWidget = sharedWidget
		if not sharedWidget is None:
			sharedWidget._texture_changed.connect(self._texture_changed)
		
		self.xRot = 0
		self.yRot = 0
		self.zRot = 0

		self.lastPos = QPoint()
		self.bgColor = QColor.fromRgb(0,0,0)
	
	def _texture_changed(self):
		self.updateGL()
	
	def setVolume(self, volume):
		self._volume = volume
		self._shape = volume._data.shape
		self._data = volume._data
	
	def enableMaxtree(self):
		self._mt = Maxtree(self._data)
		self._mainwindow._filter_controls.setEnabled(True)
	
	def maxtreeFilter(self, t1, t2):
		time.sleep(1)
		n = self._mt.volume_filter(t1, t2)
		self._data = self._mt.get_filtered_data()
		self.loadTextures()
		self.updateGL()
		return n
	
	def xRotation(self):
		return self.xRot

	def yRotation(self):
		return self.yRot

	def zRotation(self):
		return self.zRot
	
	def setXRotation(self, angle):
		angle = self.normalizeAngle(angle)
		if angle != self.xRot:
			self.xRot = angle
			self.emit(SIGNAL("xRotationChanged(int)"), angle)
			self.updateGL()

	def setYRotation(self, angle):
		angle = self.normalizeAngle(angle)
		if angle != self.yRot:
			self.yRot = angle
			self.emit(SIGNAL("yRotationChanged(int)"), angle)
			self.updateGL()

	def setZRotation(self, angle):
		angle = self.normalizeAngle(angle)
		if angle != self.zRot:
			self.zRot = angle
			self.emit(SIGNAL("zRotationChanged(int)"), angle)
			self.updateGL()
	
	def mousePressEvent(self, event):
		self.lastPos = QPoint(event.pos())
		self.sharedWidget.lowresolution = 1

	def mouseReleaseEvent(self, event):
		self.sharedWidget.lowresolution = 0
		self.updateGL()
	
	def mouseMoveEvent(self, event):
		dx = event.x() - self.lastPos.x()
		dy = event.y() - self.lastPos.y()

		if event.buttons() & Qt.LeftButton:
			self.setXRotation(self.xRot + 8 * dy)
			self.setYRotation(self.yRot + 8 * dx)
		elif event.buttons() & Qt.RightButton:
			self.setXRotation(self.xRot + 8 * dy)
			self.setZRotation(self.zRot + 8 * dx)

		self.lastPos = QPoint(event.pos())
	
	def normalizeAngle(self, angle):
		while angle < 0:
			angle += 360 * 16
		while angle > 360 * 16:
			angle -= 360 * 16
		return angle
	
	def loadTextures(self):
		s,m,n = self._shape
		
		# Prepara o volume
		'''
		texBuffer = self._volume.get_data().astype('float')
		umin, umax = texBuffer.min(), texBuffer.max()
		texBuffer = np.round(255*(texBuffer - umin)/(umax-umin)).astype('uint8')
		'''
		print self._shape, self._data.shape
		print self._data.dtype, self._data.min(), self._data.mean(), self._data.max()
		# Carrega o volume em niveis de cinza como textura 3D
		GL.glBindTexture(GL.GL_TEXTURE_3D, self.TEXID)
		GL.glTexImage3D(GL.GL_TEXTURE_3D, 0, GL.GL_LUMINANCE, s, m, n, 0,
			   GL.GL_LUMINANCE, GL.GL_UNSIGNED_BYTE, self._data.T)
		
		GL.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE)
		GL.glTexParameteri(GL.GL_TEXTURE_3D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR)
		GL.glTexParameteri(GL.GL_TEXTURE_3D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR)
		
		GL.glTexParameteri(GL.GL_TEXTURE_3D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP_TO_EDGE)
		GL.glTexParameteri(GL.GL_TEXTURE_3D, GL.GL_TEXTURE_WRAP_R, GL.GL_CLAMP_TO_EDGE)
		GL.glTexParameteri(GL.GL_TEXTURE_3D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP_TO_EDGE)
	
	def initializeGL(self):
		self.qglClearColor(self.bgColor)
		
		self.TEXID, _ = GL.glGenTextures(2)
		self.loadTextures()
		
		#GL.glEnable(GL.GL_DEPTH_TEST)
		GL.glEnable(GL.GL_CULL_FACE)
		# Seta modo de renderizacao
		GL.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_FILL)
		# habilita a texturização
		GL.glEnable(GL.GL_TEXTURE_3D)
		
		# Compile shaders
		self.shader_program = QGLShaderProgram(self.context())
		self.shader_program.addShaderFromSourceFile(QGLShader.Vertex, 'shaders/raycasting.vert')
		self.shader_program.addShaderFromSourceFile(QGLShader.Fragment, 'shaders/raycasting.frag')
		self.shader_program.link()
		self.shader_program.bind()
		
		prog = self.shader_program.programId()
		self.VERTEX = GL.glGetAttribLocation(prog,"position")
		self.TEXPOS = GL.glGetAttribLocation(prog,"texCoord0")
		self.INVMAT = GL.glGetUniformLocation(prog, "viewMatrixInv")
		self.STEP = GL.glGetUniformLocation(prog,"step")
		self.RAYDIR = GL.glGetUniformLocation(prog,"ray_direction")
		self.BGCOLOR = GL.glGetUniformLocation(prog,"bgColor")
		self.TEXTURE = GL.glGetUniformLocation(prog,"VOLUME")
		self.OPACITY = GL.glGetUniformLocation(prog,"OPACITY")
		self.COLOR = GL.glGetUniformLocation(prog,"COLOR")
	
	def paintGL(self):
		s,m,n = self._volume.get_shape(True)
		GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
		GL.glLoadIdentity()
		GL.glRotated(self.xRot / 16.0, 1.0, 0.0, 0.0)
		GL.glRotated(self.yRot / 16.0, 0.0, 1.0, 0.0)
		GL.glRotated(self.zRot / 16.0, 0.0, 0.0, 1.0)
		GL.glTranslated(-s/2, -m/2, -n/2)
		self.shader_program.bind()
		
		modelviewinv = np.linalg.inv(GL.glGetFloatv(GL.GL_MODELVIEW_MATRIX))
		passo=0.01;
		try:
			GL.glVertexAttribPointer(self.VERTEX, 3, GL.GL_FLOAT, False, 12, self._volume.get_parallelepiped())
			GL.glEnableVertexAttribArray(self.VERTEX)
			GL.glVertexAttribPointer(self.TEXPOS, 3, GL.GL_FLOAT, True, 12, self._volume.get_texture_coordinates())
			GL.glEnableVertexAttribArray(self.TEXPOS)
			GL.glUniformMatrix4fv(self.INVMAT, 1, GL.GL_FALSE, modelviewinv)
			GL.glUniform3f(self.STEP, 0.01, 0.001, self.sharedWidget.lowresolution)
			ray = np.dot(np.array([0,0,-1,0]), modelviewinv)[:3] / self._volume.get_shape(True)
			GL.glUniform3fv(self.RAYDIR, 1, ray / np.sqrt((ray**2).sum()))
			GL.glUniform3fv(self.BGCOLOR, 1, np.array(self.bgColor.getRgb())[:3]/255.0)
			GL.glActiveTexture(GL.GL_TEXTURE0)
			GL.glEnable(GL.GL_TEXTURE_3D)
			GL.glBindTexture(GL.GL_TEXTURE_3D, self.TEXID)
			GL.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE);
			GL.glUniform1i(self.TEXTURE,0)
			GL.glActiveTexture(GL.GL_TEXTURE0+1)
			GL.glEnable(GL.GL_TEXTURE_1D)
			GL.glBindTexture(GL.GL_TEXTURE_1D, self.sharedWidget.OPACITYTEXID)
			GL.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE);
			GL.glUniform1i(self.OPACITY,1)
			GL.glActiveTexture(GL.GL_TEXTURE0+2)
			GL.glEnable(GL.GL_TEXTURE_1D)
			GL.glBindTexture(GL.GL_TEXTURE_1D, self.sharedWidget.COLORTEXID)
			GL.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE);
			GL.glUniform1i(self.COLOR,2)
			GL.glDrawArrays(GL.GL_QUADS, 0, 24)
		finally:
			GL.glFlush()
			GL.glUseProgram(0)
	
	def resizeGL(self, width, height):
		GL.glViewport(0, 0, width, height)
		
		GL.glMatrixMode(GL.GL_PROJECTION)
		GL.glLoadIdentity()
		x,y = 0.8,0.8
		if width>height:
			x = (1.0*width/height) * x
		else:
			y = (1.0*height/width) * y
		GL.glOrtho(-x, x, -y, y, -25, 25)
		GL.glMatrixMode(GL.GL_MODELVIEW)

class GLSlicer(QGLWidget):
	def __init__(self, parent=None, sharedWidget=None):
		QGLWidget.__init__(self, parent, shareWidget=sharedWidget)
		self.sharedWidget = sharedWidget
	
	def __make_slices(self):
		if self.sharedWidget is None:
			raise Exception('Textures not initialized.')
	
	def initializeGL(self):
		pass
	
	def paintGL(self):
		pass
	
	def resizeGL(self, width, height):
		pass

class GLCurveEditor(QGLWidget):
	_texture_changed = Signal()
	
	def __init__(self, parent=None, sharedWidget=None):
		QGLWidget.__init__(self, parent, shareWidget=sharedWidget)
		self.sharedWidget = sharedWidget
		self._opacity_function = Opacity_Function()
		self._color_function = Color_Function()
		self.lowresolution = 0
	
	def __refresh(self):
		self.__load_opacity_function()
		self.__load_color_function()
		self.updateGL()
		self._texture_changed.emit()
	
	def __get_handler(self, y):
		if y>=0 and y<=1:
			self._handler = self._opacity_function
		elif y>=1.1 and y<=1.3:
			self._handler = self._color_function
		else:
			self._handler = None
		return self._handler
	
	def mouseDoubleClickEvent(self, event):
		x, y = self.__convert_coord(event.pos())
		
		handler = self.__get_handler(y)
		if handler is None:
			return False
		
		if event.buttons() == Qt.LeftButton:
			if (handler._current_knot is None and handler.create_knot(x, y)) or (handler==self._color_function and handler.select_color()):
				self.__refresh()
		
		if event.buttons() == Qt.RightButton:
			if handler.erase_knot(x, y):
				self.__refresh()
		
		return True
	
	def mousePressEvent(self, event):
		self.lowresolution = 1
		self._opacity_function._current_knot = None
		self._color_function._current_knot = None
		x, y = self.__convert_coord(event.pos())
		
		handler = self.__get_handler(y)
		if handler is None:
			return False
		
		if handler.select_knot(x, y):
			self.__refresh()
	
	def mouseReleaseEvent(self, event):
		self.lowresolution = 0
		self.__refresh()
	
	def mouseMoveEvent(self, event):
		if event.buttons() == Qt.LeftButton:
			x, y = self.__convert_coord(event.pos())
			
			handler = self.__get_handler(y)
			if handler is None:
				return False
			
			handler.move_selected_knot(x, y)
			self.__refresh()
	
	def __convert_coord(self, pos):
		x, y = pos.toTuple()
		x, y = 1.0*x/self.width(), 1 - 1.0*y/self.height()
		x, y = x*1.2 - 0.1, y*1.8 - 0.2
		return (x,y)
	
	def __load_opacity_function(self):
		opacity = np.round(self._opacity_function.get(256)*255).astype('uint8')
		
		# Carrega o volume em niveis de cinza como textura 3D
		GL.glBindTexture(GL.GL_TEXTURE_1D, self.OPACITYTEXID)
		GL.glTexImage1D(GL.GL_TEXTURE_1D, 0, GL.GL_LUMINANCE, 256, 0,
			   GL.GL_LUMINANCE, GL.GL_UNSIGNED_BYTE, opacity)

		GL.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE)
		GL.glTexParameteri(GL.GL_TEXTURE_1D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR)
		GL.glTexParameteri(GL.GL_TEXTURE_1D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR)
		GL.glTexParameteri(GL.GL_TEXTURE_1D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP_TO_EDGE)
		GL.glTexParameteri(GL.GL_TEXTURE_1D, GL.GL_TEXTURE_WRAP_R, GL.GL_CLAMP_TO_EDGE)
		GL.glTexParameteri(GL.GL_TEXTURE_1D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP_TO_EDGE)
	
	def __load_color_function(self):
		color = np.round(self._color_function.get(256)*255).astype('uint8')
		
		# Carrega o volume em niveis de cinza como textura 3D
		GL.glBindTexture(GL.GL_TEXTURE_1D, self.COLORTEXID)
		GL.glTexImage1D(GL.GL_TEXTURE_1D, 0, GL.GL_RGB, 256, 0,
			   GL.GL_RGB, GL.GL_UNSIGNED_BYTE, color)

		GL.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE)
		GL.glTexParameteri(GL.GL_TEXTURE_1D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR)
		GL.glTexParameteri(GL.GL_TEXTURE_1D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR)
		GL.glTexParameteri(GL.GL_TEXTURE_1D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP_TO_EDGE)
		GL.glTexParameteri(GL.GL_TEXTURE_1D, GL.GL_TEXTURE_WRAP_R, GL.GL_CLAMP_TO_EDGE)
		GL.glTexParameteri(GL.GL_TEXTURE_1D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP_TO_EDGE)
	
	def initializeGL(self):
		self.qglClearColor(QColor(16,0,8))
		self.OPACITYTEXID, self.COLORTEXID = GL.glGenTextures(2)
		self.__load_opacity_function()
		self.__load_color_function()
		
		GL.glEnable(GL.GL_CULL_FACE)
		GL.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_FILL)
	
	def paintGL(self):
		GL.glClear(GL.GL_COLOR_BUFFER_BIT)
		self._opacity_function.draw()
		self._color_function.draw()
	
	def resizeGL(self, width, height):
		GL.glViewport(0, 0, width, height)
		
		GL.glMatrixMode(GL.GL_PROJECTION)
		GL.glLoadIdentity()
		GL.glOrtho(-0.1, 1.1, -0.2, 1.6, -1, 1)
		GL.glMatrixMode(GL.GL_MODELVIEW)

class F_Annotation(QFrame):
	def __init__(self, image, parent=None):
		from control import ImageControl
		QFrame.__init__(self, parent)
		layout = QHBoxLayout()
		scene = QGraphicsScene()
		#scene.addPixmap(QPixmap.fromImage(image.getImage()))
		#img = TargetableImage(image)
		#eli = QGraphicsEllipseItem(0,0,2,2,parent=img)
		scene.addItem(image)
		viewer = QGraphicsView()
		layout.addWidget(viewer, stretch=1)
		self.setLayout(layout)
		viewer.setScene(scene)
		viewer.show()
		#imgC = ImageControl()
		image._mousePress.connect(ImageControl.mousePress)
